Class: PatientHttp::SolidQueue::GcLock

Inherits:
Record
  • Object
show all
Defined in:
lib/patient_http/solid_queue/gc_lock.rb

Overview

Active Record model for distributed garbage collection locking.

Ensures only one process runs orphan detection at a time. The last_gc_at column records when GC was last successfully completed, allowing processes to skip GC attempts if another process ran GC recently.